#include "ofApp.h"

//--------------------------------------------------------------
// setup -- Setup our application
//--------------------------------------------------------------
void ofApp::setup(){
    // AUDIO...

    // Check out the available audio devices
    // soundStream.listDevices();

    // Setup the sound stream
    soundStream.setup(this, 0, MY_CHANNELS, MY_SRATE, MY_BUFFERSIZE, MY_NBUFFERS);

    // Resize and initialize left and right buffers...
    left.resize( MY_BUFFERSIZE, 0 );
    right.resize( MY_BUFFERSIZE, 0 );

    // FFT stuff for Chuck_FFT

    // To begin, set the FFT buffer to zeros
    memset( fftBuffer, 0, sizeof(complex)*MY_BUFFERSIZE/2 );

    // Create a (mono) window buffer, for use before FFT
    hamming( my_window, MY_BUFFERSIZE);


    // Setup chimes and initialize variables
    chimes.loadSound("chimes.wav");
    chimes.setLoop(true);
    chimes.play();
    chimeVol = 0.0;
    power.resize(20);
    curPow=0;
    mainangle=0;


    // VIDEO...

    // Set booleans & title
    fulscr=false;
    cheat=false;
    trailson=false;
    revangle=false;
    ofSetWindowTitle("alexsounder v0.1");

    // Sync video refresh rate
    ofSetVerticalSync(true);

    // Set background color
    ofBackground(235, 235, 235);

    // Smooth edges
    ofEnableSmoothing();
    ofEnableDepthTest();

    // Font stuff
    myfont.loadFont("DejaVuSerif-Italic.ttf",24);
    textalpha=255;

    // Setup freq domain stuff
    fbinshist.resize(11);
    for( unsigned int i = 0; i < fbinshist.size() ; i++ )
    {
        fbinshist[i].resize(100);
    }

    fbinspow.resize(11);

    fbinsavg.resize(11);
    for( unsigned int i = 0; i < fbinsavg.size() ; i++ )
    {
        fbinsavg[i].resize(100);
    }

    // Setup rotation stuff
    miniangles.resize(9);
    miniacceldir.resize(9);
    for( unsigned int i = 0; i < miniangles.size() ; i++ )
    {
        miniangles[i]=ofRandom(-2,2);
        miniacceldir[i]=ofSign(miniangles[i]);
    }

    // Print Instructions
    ofLog(OF_LOG_NOTICE, "ALEXSOUNDER v0.1 by AIDAN MEACHAM\n Blow on the microphone and drift away...\n Controls:\n [f]: toggle fullscreen\n [a]: toggle readouts\n [r]: reverse accel direction\n [c]: circle trails\n [esc]: quit");
}

//--------------------------------------------------------------
void ofApp::update(){

    // Update the ChucK_FFT -- right channel only

    apply_window( &right[0], my_window, MY_BUFFERSIZE ); // Window the buffer

    rfft( &right[0], MY_BUFFERSIZE/2, FFT_FORWARD ); // Compute the FFT in place

    memcpy( fftBuffer, &right[0], sizeof(complex)*MY_BUFFERSIZE/2); // Copy it


    // Update chime junk

    // Reset this update's power
    power[curPow%power.size()]=0.0;

    // Get this update's power
    for( unsigned int i = 0; i < MY_BUFFERSIZE/2; i++)
    {
        power[curPow%power.size()]+=cmp_abs(fftBuffer[i]);
    }

    // Reset sum of all power
    powSum=0.0;

    // Sum all stored powers
    for( unsigned int i = 0; i < power.size() ; i++)
    {
        powSum=powSum+power[i];
    }

    // Create average
    avgPow=powSum/power.size();

    // Define chime volume behavior
    if(chimeVol<avgPow){
        chimeVol=avgPow;
    }
    else{
        chimeVol=chimeVol*.995;
    }

    // Set chime volume
    chimes.setVolume(chimeVol);

    // Increment current power index
    curPow++;

    // Fade out instructions
    if(chimeVol>.4){
        textalpha+=-1;
    }

    // Get main acceleration vector
    accel=chimeVol*4;

    // Reverse if selected
    if(revangle){
        accel=-accel;
    }

    // Define main "motor" angle
    mainangle+=3+accel;

    // Define smaller joint angles
    for( unsigned int i = 0; i < miniangles.size() ; i++)
    {
        miniangles[i]=miniangles[i]+accel*miniacceldir[i]*.01;
    }

    // Reverse accel direction if limit exceeded
    if(abs(miniangles[0])>5){miniacceldir[0]=-miniacceldir[0];}
    if(abs(miniangles[1])>10){miniacceldir[1]=-miniacceldir[1];}
    if(abs(miniangles[2])>7){miniacceldir[2]=-miniacceldir[2];}
    if(abs(miniangles[3])>5){miniacceldir[3]=-miniacceldir[3];}
    if(abs(miniangles[4])>5){miniacceldir[4]=-miniacceldir[4];}
    if(abs(miniangles[5])>8){miniacceldir[5]=-miniacceldir[5];}
    if(abs(miniangles[6])>6){miniacceldir[6]=-miniacceldir[6];}
    if(abs(miniangles[7])>9){miniacceldir[7]=-miniacceldir[7];}
    if(abs(miniangles[8])>7){miniacceldir[8]=-miniacceldir[8];}

    //ofLog(OF_LOG_NOTICE, "fftBuffer: " + ofToString(1500*cmp_abs(fftBuffer[(6*(MY_BUFFERSIZE/2)/fbins.size())])));
    //ofLog(OF_LOG_NOTICE, "power: " + ofToString(power) + " volume: " + ofToString(chimeVol) + " avgpow: " + ofToString(avgPow));
}

//--------------------------------------------------------------
void ofApp::draw(){

    // Bin power levels by frequency
    for( unsigned int i = 0; i < fbinspow.size() ; i++ )
    {
        fbinspow[i]=0;

        for( unsigned int j = 0; j < MY_BUFFERSIZE/2 ; j++ )
        {
            if(i*(MY_BUFFERSIZE/2)/fbinspow.size()<j && j<(i+1)*(MY_BUFFERSIZE/2)/fbinspow.size())
            {
                fbinspow[i]=fbinspow[i]+255*1500*cmp_abs(fftBuffer[j]);
            }
        }
    }

    // Save binned power levels to history
    for( unsigned int j = 0; j < fbinshist.size() ; j++ )
    {
        fbinshist[j][curPow%fbinshist.size()]=fbinspow[j];
    }

    // Calculate time-averaged binned power levels
    for( unsigned int i = 0; i < fbinshist.size() ; i++ )
    {
        fbinsavg[i][curPow%fbinsavg[i].size()]=0;

        for( unsigned int j = 0; j < fbinshist[i].size() ; j++ )
        {
            fbinsavg[i][curPow%fbinsavg[i].size()]=fbinsavg[i][curPow%fbinsavg[i].size()]+fbinshist[i][j];
        }

        fbinsavg[i][curPow%fbinsavg[i].size()]=fbinsavg[i][curPow%fbinsavg[i].size()]/fbinshist[i].size();
    }

    // DRAW FPS AND ASSIGNMENT

    if(cheat){

        // Draw FPS
        ofDrawBitmapString(ofToString(ofGetFrameRate())+"fps", 10, 15);

        ofPushStyle();
        ofPushMatrix();

            // Draw time domain

            ofTranslate(10, 30, 0);

            ofSetColor(245, 58, 135);
            ofNoFill();

            ofBeginShape();
            for (unsigned int i = 0; i < left.size(); i++){
                ofVertex(i*75/left.size(),
                         40 * left[i],
                         0);
            }
            ofEndShape(false); // End line. Don't connect first and last points.

        ofPopMatrix();
        ofPopStyle();

        ofPushStyle();
        ofPushMatrix();

            // Draw freq domain

            ofTranslate(10, 45, 0);

            ofSetColor(ofColor::green);
            ofNoFill();

            ofBeginShape();
            for( unsigned int i = 0; i < MY_BUFFERSIZE/4; i++){

                ofVertex( 4*i*75 / (float)MY_BUFFERSIZE,
                         -1500 * cmp_abs(fftBuffer[i]),
                         0);

            }
            ofEndShape(false); // End line. Don't connect first and last points.

        ofPopMatrix();
        ofPopStyle();

    }

    // DRAW INSTRUCTIONS

    ofPushStyle();

        ofSetColor(255,255,255,textalpha);

        myfont.drawString("blow on the microphone and drift away...", 100, 35);

    ofPopStyle();

    // MEASURE SCREEN STUFF

    int wh = ofGetWindowHeight();
    int ww = ofGetWindowWidth();
    int sqr = min(wh,ww);

    int dim = sqr;
    int depth = -700;

    // DRAW FLOOR

    ofPushStyle();
    ofPushMatrix();

        ofTranslate(0,wh,100);

        ofRotateX(90);

        ofEnableAlphaBlending();

        for( int i = 0; i < 500; i++)
        {
            ofSetColor(250,250,250,255*(1-(i/500.0))); //
            ofRect(-5000,-10*i,dim+10000,-10);
        }

        ofDisableAlphaBlending();

    ofPopMatrix();
    ofPopStyle();

    // DRAW MOBILE

    ofPushStyle();
    ofPushMatrix();

        // restrict to square

        if(wh>ww){
            ofTranslate(0,(wh-ww)/2+ww,depth);
        }
        else if(ww>wh){
            ofTranslate((ww-wh)/2,wh,depth);
        }
        else{
            ofTranslate(0,sqr,depth);
        }

        // draw base circle
        ofPushStyle();
        ofPushMatrix();

            ofTranslate(0,-1,0);
            ofRotateX(90);
            ofSetColor(ofColor::black);

            ofCircle(0,0,dim*.07);

        ofPopMatrix();
        ofPopStyle();

        // draw stand
        ofSetLineWidth(3);
        ofSetColor(ofColor::black);
        ofNoFill();

        ofBeginShape();
        ofBezier(0.0,0.0,0.0,
                 0.0,-dim,0.0,
                 dim*.52,-dim,-depth/2,
                 dim*.52,-.9*dim,-depth/2);
        ofEndShape();

        // draw mobile
        ofEnableAlphaBlending();

        int ncopies = 1;
        if(trailson){
            ncopies = 45;
        }

        for( int i = 0; i < ncopies ; i++ ){
            ofApp::drawMobile(i,ncopies);
        }

        ofDisableAlphaBlending();
        // end mobile
    ofPopMatrix();
    ofPopStyle();
    // end stand
}

void ofApp::drawMobile(int i, int ncopies){

    int wh = ofGetWindowHeight();
    int ww = ofGetWindowWidth();
    int sqr = min(wh,ww);

    int dim = sqr;
    int depth = -700;

ofPushMatrix();

    ofTranslate(.52*dim,-.9*dim,-depth/2);
    ofRotateY(mainangle * .02 - 8 * i);

    ofBeginShape();
        ofVertex(0,0);
        ofVertex(0,.174*dim);
    ofEndShape(false);

    ofPushMatrix();

        ofTranslate(0,0.174*dim);

        ofPushMatrix();

            ofTranslate(-.074*dim,.045*dim);

            ofBeginShape();
            ofBezier(0.0,0.0,
                     0.110*dim,-0.078*dim,
                     0.356*dim,-0.126*dim,
                     0.464*dim,-0.137*dim);
            ofEndShape();

            ofSetColor(fbinsavg[10][curPow%fbinsavg[0].size()-i],0,0,255*((ncopies-(float)i)/ncopies));
            ofFill();
            ofCircle(0.464*dim,-0.137*dim,0.046*dim);
            ofSetColor(0,0,0,255*((ncopies-(float)i)/ncopies));
            ofNoFill();

            //rotate
            ofRotateY(miniangles[0]);

            ofPushMatrix();

                ofTranslate(-.062*dim,.025*dim);

                ofBeginShape();
                ofBezier(0.0,0.0,
                         0.101*dim,-0.056*dim,
                         0.360*dim,-0.027*dim,
                         0.4455*dim,-0.0075*dim);
                ofEndShape();

                ofSetColor(fbinsavg[9][curPow%fbinsavg[0].size()-i],0,0,255*((ncopies-(float)i)/ncopies));
                ofFill();
                ofCircle(0.4455*dim,-0.0075*dim,0.0365*dim);
                ofSetColor(0,0,0,255*((ncopies-(float)i)/ncopies));
                ofNoFill();

                //rotate
                ofRotateY(miniangles[1]);

                ofPushMatrix();

                    ofTranslate(-.003*dim,.0585*dim);

                    ofBeginShape();
                    ofBezier(0.0,0.0,
                             0.003*dim,-0.0225*dim,
                             -0.010*dim,-0.1335*dim,
                             -0.021*dim,-0.1775*dim);
                    ofEndShape();

                    ofSetColor(fbinsavg[8][curPow%fbinsavg[0].size()-i],0,0,255*((ncopies-(float)i)/ncopies));
                    ofFill();
                    ofCircle(-0.021*dim,-0.1775*dim,0.0185*dim);
                    ofSetColor(0,0,0,255*((ncopies-(float)i)/ncopies));
                    ofNoFill();

                    //rotate
                    ofRotateY(miniangles[2]);

                    ofPushMatrix();

                        ofTranslate(-.0005*dim,.034*dim);

                        ofBeginShape();
                        ofBezier(0.0,0.0,
                                 0.000*dim,-0.0365*dim,
                                 -0.0295*dim,-0.1345*dim,
                                 -0.0445*dim,-0.1385*dim);
                        ofEndShape();

                        ofSetColor(fbinsavg[7][curPow%fbinsavg[0].size()-i],0,0,255*((ncopies-(float)i)/ncopies));
                        ofFill();
                        ofCircle(-0.0445*dim,-0.1385*dim,0.019*dim);
                        ofSetColor(0,0,0,255*((ncopies-(float)i)/ncopies));
                        ofNoFill();

                        //rotate
                        ofRotateY(miniangles[3]);

                        ofPushMatrix();

                            ofTranslate(.0355*dim,.0235*dim);

                            ofBeginShape();
                            ofBezier(0.0,0.0,
                                     0.007*dim,-0.037*dim,
                                     -0.2025*dim,-0.0405*dim,
                                     -0.2905*dim,-0.0345*dim);
                            ofEndShape();

                            ofSetColor(fbinsavg[6][curPow%fbinsavg[0].size()-i],0,0,255*((ncopies-(float)i)/ncopies));
                            ofFill();
                            ofCircle(-0.2905*dim,-0.0345*dim,0.027*dim);
                            ofSetColor(0,0,0,255*((ncopies-(float)i)/ncopies));
                            ofNoFill();

                            //rotate
                            ofRotateY(miniangles[4]);

                            ofPushMatrix();

                                ofTranslate(.080*dim,.029*dim);

                                ofBeginShape();
                                ofBezier(0.0,0.0,
                                         -0.049*dim,-0.061*dim,
                                         -0.358*dim,0.0035*dim,
                                         -0.432*dim,0.025*dim);
                                ofEndShape();

                                ofSetColor(fbinsavg[5][curPow%fbinsavg[0].size()-i],0,0,255*((ncopies-(float)i)/ncopies));
                                ofFill();
                                ofCircle(-0.432*dim,0.025*dim,0.028*dim);
                                ofSetColor(0,0,0,255*((ncopies-(float)i)/ncopies));
                                ofNoFill();

                                //rotate
                                ofRotateY(miniangles[5]);

                                ofPushMatrix();

                                    ofTranslate(.091*dim,.0145*dim);

                                    ofBeginShape();
                                    ofBezier(0.0,0.0,
                                             -0.045*dim,-0.050*dim,
                                             -0.298*dim,0.0515*dim,
                                             -0.347*dim,0.0785*dim);
                                    ofEndShape();

                                    ofSetColor(fbinsavg[4][curPow%fbinsavg[0].size()-i],0,0,255*((ncopies-(float)i)/ncopies));
                                    ofFill();
                                    ofCircle(-0.347*dim,0.0785*dim,0.02625*dim);
                                    ofSetColor(0,0,0,255*((ncopies-(float)i)/ncopies));
                                    ofNoFill();

                                    //rotate
                                    ofRotateY(miniangles[6]);

                                    ofPushMatrix();

                                        ofTranslate(-.012*dim,.0415*dim);

                                        ofBeginShape();
                                        ofBezier(0.0,0.0,
                                                 0.004*dim,-0.038*dim,
                                                 0.051*dim,-0.123*dim,
                                                 0.0695*dim,-0.1465*dim);
                                        ofEndShape();

                                        ofSetColor(fbinsavg[3][curPow%fbinsavg[0].size()-i],0,0,255*((ncopies-(float)i)/ncopies));
                                        ofFill();
                                        ofCircle(0.0695*dim,-0.1465*dim,0.01875*dim);
                                        ofSetColor(0,0,0,255*((ncopies-(float)i)/ncopies));
                                        ofNoFill();

                                        //rotate
                                        ofRotateY(miniangles[7]);

                                        ofPushMatrix();

                                            ofTranslate(-.031*dim,.0425*dim);

                                            ofBeginShape();
                                            ofBezier(0.0,0.0,
                                                     0.0025*dim,-0.014*dim,
                                                     0.078*dim,-0.095*dim,
                                                     0.1175*dim,-0.1185*dim);
                                            ofEndShape();

                                            ofSetColor(fbinsavg[2][curPow%fbinsavg[0].size()-i],0,0,255*((ncopies-(float)i)/ncopies));
                                            ofFill();
                                            ofCircle(0.1175*dim,-0.1185*dim,0.01875*dim);
                                            ofSetColor(0,0,0,255*((ncopies-(float)i)/ncopies));
                                            ofNoFill();

                                            //rotate
                                            ofRotateY(miniangles[8]);

                                            ofPushMatrix();

                                                ofTranslate(-.0645*dim,.0495*dim);

                                                ofBeginShape();
                                                ofBezier(0.0,0.0,
                                                         0.0375*dim,-0.033*dim,
                                                         0.0995*dim,-0.0755*dim,
                                                         0.131*dim,-0.0825*dim);
                                                ofEndShape();

                                                ofSetColor(fbinsavg[0][curPow%fbinsavg[0].size()-i],0,0,255*((ncopies-(float)i)/ncopies));
                                                ofFill();
                                                ofCircle(0,0,0.01875*dim);

                                                ofSetColor(fbinsavg[1][curPow%fbinsavg[0].size()-i],0,0,255*((ncopies-(float)i)/ncopies));
                                                ofCircle(0.131*dim,-0.0825*dim,0.01875*dim);
                                                ofSetColor(0,0,0,255*((ncopies-(float)i)/ncopies));
                                                ofNoFill();

                                            ofPopMatrix();

                                        ofPopMatrix();

                                    ofPopMatrix();

                                ofPopMatrix();

                            ofPopMatrix();

                        ofPopMatrix();

                    ofPopMatrix();

                ofPopMatrix();

            ofPopMatrix();

        ofPopMatrix();

    ofPopMatrix();

ofPopMatrix();
}

//--------------------------------------------------------------
void ofApp::audioIn(float * input, int bufferSize, int nChannels){

    // Write incoming audio to buffer. Note: samples are interleaved.
    for( int i = 0; i < bufferSize; i++){
        left[i]		= input[i*2]*0.5;
        right[i]	= input[i*2+1]*0.5;
    }

}

//--------------------------------------------------------------
void ofApp::keyPressed(int key){
    if(key=='f'){
        fulscr=!fulscr;
    }

    if(key=='a'){
        cheat=!cheat;
    }

    if(key=='r'){
        revangle=!revangle;
    }

    if(key=='c'){
        trailson=!trailson;
    }

    if(key==27){
        fulscr=false;
    }

    ofSetFullscreen(fulscr);

}

//--------------------------------------------------------------
void ofApp::keyReleased(int key){

}

//--------------------------------------------------------------
void ofApp::mouseMoved(int x, int y ){

}

//--------------------------------------------------------------
void ofApp::mouseDragged(int x, int y, int button){

}

//--------------------------------------------------------------
void ofApp::mousePressed(int x, int y, int button){

}

//--------------------------------------------------------------
void ofApp::mouseReleased(int x, int y, int button){

}

//--------------------------------------------------------------
void ofApp::windowResized(int w, int h){

}

//--------------------------------------------------------------
void ofApp::gotMessage(ofMessage msg){

}

//--------------------------------------------------------------
void ofApp::dragEvent(ofDragInfo dragInfo){

}
